var prog_tunnel, prog_text;
var tex_stones, tex_normal, tex_text;

const sides = 24;
const segm = 20;
const ring_height = 0.5;
const text_period = 13.0;
const text_speed = 2.2;

var prev_text_pos;


function init_gl()
{
	if(!glass_init("glcanvas", "console")) {
		return;
	}
	glass_display_func(disp);
	glass_reshape_func(reshape);
	/*glass_mouse_func(mouse);
	glass_motion_func(motion);*/

	if(!(prog_tunnel = load_program("tunnel.v.glsl", "tunnel.p.glsl"))) {
		return;
	}
	if(!(prog_text = load_program("text.v.glsl", "text.p.glsl"))) {
		return;
	}
	tex_stones = load_texture("stonewall.jpg");
	tex_normal = load_texture("stonewall_normal.jpg");
	tex_text = load_texture("text.png");

	gl.enable(gl.DEPTH_TEST);
	gl.enable(gl.CULL_FACE);

	window.onresize = win_resized;

	glass_start();
}

function disp(msec)
{
	gl.clearColor(0.01, 0.01, 0.01, 1.0);
	gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);

	var sec = msec / 1000.0;
	var z = ring_height * segm;
	var pan = worm(sec, z);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(-pan[0], -pan[1], -1.1 * ring_height * segm);
	/*glTranslatef(-5, 2, 4);
	glRotatef(40.0, 1.0, 0.0, 0.0);
	glRotatef(50.0, 0.0, 1.0, 0.0);*/

	draw_tunnel(sec);

	gl.depthMask(false);
	var text_line = Math.floor((text_speed * sec) / text_period);
	for(var i=0; i<15; i++) {
		var tpos = calc_text_pos(sec - i * 0.01);
		draw_text(text_line, tpos, 1.5 / i);
	}
	//draw_text(sec);
	gl.depthMask(true);
}

function draw_tunnel(t)
{
	gl.useProgram(prog_tunnel);

	var loc = gl.getUniformLocation(prog_tunnel, "t");
	if(loc != -1) {
		gl.uniform1f(loc, t);
	}
	loc = gl.getUniformLocation(prog_tunnel, "tex");
	if(loc != -1) {
		gl.uniform1i(loc, 0);
	}
	loc = gl.getUniformLocation(prog_tunnel, "tex_norm");
	if(loc != -1) {
		gl.uniform1i(loc, 1);
	}
	loc = gl.getUniformLocation(prog_tunnel, "light_pos");
	if(loc != -1) {
		var pos = calc_text_pos(t);
		gl.uniform4f(loc, pos[0], pos[1], pos[2], 1.0);
	}

	loc = gl.getAttribLocation(prog_tunnel, "attr_tangent");

	gl.activeTexture(gl.TEXTURE1);
	gl.bindTexture(gl.TEXTURE_2D, tex_normal);

	gl.activeTexture(gl.TEXTURE0);
	gl.bindTexture(gl.TEXTURE_2D, tex_stones);

	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();
	//glTranslatef(0, -t * 1.5, 0);
	glTranslatef(0, -t * 0.5, 0);

	glBegin(GL_QUADS);
	glColor3f(1.0, 1.0, 1.0);

	var du = 1.0 / sides;
	var dv = 1.0 / segm;

	const uoffs = [0.0, 0.0, 1.0, 1.0];
	const voffs = [0.0, 1.0, 1.0, 0.0];

	for(var i=0; i<segm; i++) {
		var zp = ring_height * (i - 1);
		var z0 = ring_height * i;
		var z1 = ring_height * (i + 1);

		var v0 = i / segm;
		var v1 = (i + 1) / segm;

		var trans_zp = worm(t, zp);
		var trans_z0 = worm(t, z0);
		var trans_z1 = worm(t, z1);

		for(var j=0; j<sides; j++) {
			for(var k=0; k<4; k++) {
				var u = (j + uoffs[k]) * du;
				var v = (i + voffs[k]) * dv;

				tunnel_vertex(u, v, du, dv, loc, t);
			}
		}
	}


	/*glNormal3f(0, 1, 0);
	glVertexAttrib3f(loc, 1.0, 0.0, 0.0);
	glTexCoord2f(0, 10);
	glVertex3f(-10, -1, 10);
	glTexCoord2f(10, 10);
	glVertex3f(10, -1, 10);
	glTexCoord2f(10, 0);
	glVertex3f(10, -1, -10);
	glTexCoord2f(0, 0);
	glVertex3f(-10, -1, -10);*/
	glEnd();

	gl.activeTexture(gl.TEXTURE1);
	gl.bindTexture(gl.TEXTURE_2D, null);
	gl.activeTexture(gl.TEXTURE0);
	gl.bindTexture(gl.TEXTURE_2D, null);
}


var pos = new Float32Array(3);
var pos_du = new Float32Array(3);
var pos_dv = new Float32Array(3);
var dfdu = new Float32Array(3);
var dfdv = new Float32Array(3);
var norm = new Float32Array(3);

function tunnel_vertex(u, v, du, dv, tloc, t)
{
	var theta = 2.0 * Math.PI * u;
	var theta1 = 2.0 * Math.PI * (u + du);

	var x = Math.cos(theta);
	var y = Math.sin(theta);
	var x1 = Math.cos(theta1);
	var y1 = Math.sin(theta1);
	var z = v / dv * ring_height;
	var z1 = (v + dv) / dv * ring_height;

	var trans_z = worm(t, z);
	var trans_z1 = worm(t, z1);

	v3_cons(pos, x + trans_z[0], y + trans_z[1], z);
	v3_cons(pos_du, x1 + trans_z[0], y1 + trans_z[1], z);
	v3_cons(pos_dv, x + trans_z1[0], y + trans_z1[1], z1);

	v3_sub(dfdu, pos_du, pos);
	v3_sub(dfdv, pos_dv, pos);
	v3_cross(norm, dfdv, dfdu);

	glVertexAttrib3f(tloc, dfdu[0], dfdu[1], dfdu[2]);
	glNormal3f(norm[0], norm[1], norm[2]);
	glTexCoord2f(u * 2.0, v * 4.0);
	glVertex3f(pos[0], pos[1], pos[2]);
}

function calc_text_pos(sec)
{
	//var t = (1.87 * sec);
	var t = (text_speed * sec);
	var z = t % text_period;
	var pan = worm(sec, z);

	return [pan[0], pan[1], z + ring_height];
}

function draw_text(idx, tpos, alpha)
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(tpos[0], tpos[1], tpos[2]);

	gl.enable(gl.BLEND);
	//gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);
	gl.blendFunc(gl.SRC_ALPHA, gl.ONE);

	gl.useProgram(prog_text);

	var loc = gl.getUniformLocation(prog_text, "idx");
	if(loc != -1) {
		gl.uniform1f(loc, idx);
	}

	gl.bindTexture(gl.TEXTURE_2D, tex_text);

	glBegin(GL_QUADS);
	glColor4f(1.0, 1.0, 1.0, alpha);

	glTexCoord2f(0, 1);
	glVertex3f(-1, -0.2, 0);

	glTexCoord2f(1, 1);
	glVertex3f(1, -0.2, 0);

	glTexCoord2f(1, 0);
	glVertex3f(1, 0.2, 0);

	glTexCoord2f(0, 0);
	glVertex3f(-1, 0.2, 0);
	glEnd();

	gl.bindTexture(gl.TEXTURE_2D, null);

	gl.disable(gl.BLEND);

	glPopMatrix();
}

function worm(t, z)
{
	var x = Math.sin(t) + Math.cos(t + z) + Math.sin(t * 2.0 + z) / 2.0;
	var y = Math.cos(t) + Math.sin(t + z) + Math.cos(t * 2.0 + z) / 2.0;

	return [x * 0.5, y * 0.5];
	//return [0.0, 0.0];
}

function reshape(x, y)
{
	logmsg("reshape(" + x + ", " + y + ")\n");
	gl.viewport(0, 0, x, y);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, x / y, 0.5, 500.0);
}

/*var prev_x = -1, prev_y = -1;
var bnstate = [false, false, false];

function mouse(bn, state, x, y)
{
	bnstate[bn] = state;
	if(state) {
		prev_x = x;
		prev_y = y;

		clearTimeout(timeout);
		rotate_auto = false;
	} else {
		prev_x = prev_y = -1;

		timeout = setTimeout(function() { rotate_auto = true; }, 5000);
	}
}

function motion(x, y)
{
	if(prev_x != -1) {
		cam_theta += (x - prev_x) * 0.5;
		cam_phi += (y - prev_y) * 0.5;

		if(cam_phi < -max_phi) cam_phi = -max_phi;
		if(cam_phi > max_phi) cam_phi = max_phi;

		prev_x = x;
		prev_y = y;
	}
}*/

function toggle_fullscreen()
{
	glass_fullscreen();

	clearTimeout(timeout);
	rotate_auto = true;
}

function win_resized()
{
	if(fullscr) {
		glass_resize(window.innerWidth, window.innerHeight);
	}
}
